close all;
clear all;

%% Read the initial image and define graph creation parameters

% Sample image file
path_to_file = 'BSDS500-selected_images/test/';
filename = '196027.jpg';
fileformat = 'jpg';

input = imread(strcat(path_to_file, filename), fileformat);
image_color = im2double(input);
image = rgb2gray(image_color);

[height, width] = size(image);

%% Create a graph through WATERSHED segmentation and define color image function on it

[L, I, indices, ~, al, c, degs, tri, n] = graph_watershed_delaunay(image);
cform = makecform('srgb2lab');
I_rgb = cat(2, image_color(indices), image_color(indices + height * width),...
    image_color(indices + 2 * height * width));
I_lab = applycform(I_rgb, cform);
ranges_lab_channels = max(I_lab) - min(I_lab);
I_lab_normalized = I_lab / median(ranges_lab_channels);

%% Initial contour: rectangle

contour_internal = abs(c(:, 1) - 0.52 * width) < 0.25 * width &...
    abs(c(:, 2) - 0.55 * height) < 0.4 * height;

%% Global parameters and initialization of the algorithm on the graph

% Choice of gradient and curvature smoothing filters.
fil = [0, 0];

% Curvature approximation method.
curv_approx = 0;

% Total iterations.
iter = 2000;

% Time step.
dt = 0.05;

% Length regularization parameter.
mu = 0.5;

% Area regularization parameter.
nu = 0;

% Intensity variation weights.
lambda_1 = 1;
lambda_2 = 1;

% Compute the fundamental geometric quantities of the graph essential for
% gradient and curvature calculations
[edgevectors, lengths, right_lengths, angles, omegas, index] =...
    edgevectors_angles(n, al, c(:, 1:2), degs);

% Space step.
epsilon = median(median(right_lengths(right_lengths > 0)));

% Initialize the embedding function as the signed distance transform from the
% initial contour.
Phi = dist_given_contour(contour_internal, c, al, n, degs);

% Modify signed distance transform.
Phi = Phi/min(width, height);

% Plot initial contour.
figure;
plot(c(:, 1), height - c(:, 2), 'o', 'MarkerSize', 2);
hold on;
plot(c(Phi > 0, 1), height - c(Phi > 0, 2), 'o',...
    'MarkerEdgeColor', 'k',...
    'MarkerFaceColor', 'r',...
    'MarkerSize', 3);
hold off;
axis equal;

%% Main iteration stage of the algorithm

stages = 100;

for i=1:iter
    Phi = acwe_iter_color(Phi, I_lab_normalized, epsilon, mu, nu, lambda_1, lambda_2, dt,...
        edgevectors, lengths, angles, omegas, right_lengths, index, n, al,...
        degs, fil, curv_approx);
    
    if ~mod(i, stages)
        figure;
        plot(c(Phi <= 0, 1), height - c(Phi <= 0, 2), 'o', 'MarkerSize', 2);
        hold on;
        plot(c(Phi > 0, 1), height - c(Phi > 0, 2), 'o',...
            'MarkerEdgeColor', 'r',...
            'MarkerFaceColor', 'r',...
            'MarkerSize', 2);
        hold off;
        axis equal;
    end
end

% Final segmentation of graph overlaid on the image.
figure;
imshow(image);
hold on;
plot(c(Phi <= 0, 1), c(Phi <= 0, 2), '.');
plot(c(Phi > 0, 1), c(Phi > 0, 2), 'r.');
h1 = tightfig(gcf);

% Determine and plot boundary of segmented objects.
boundary_color = [1, 1, 1];
image_color_with_boundary = plot_contour_on_color_image(Phi, L, image_color,...
    boundary_color);
iptsetpref('ImshowBorder', 'tight');
figure; imshow(image_color_with_boundary);
