% Creating the graph
close all;
clear all;

n = 7000;
a = 1/3;
b = 3/5;
r = b*n^(-a);
[~, al, c, degs] = grgall_small_lists(n, r);

% Embedding function's parameters
x0 = -0.25;
y0 = 0.5;
c_x = 0.4;
c_y = 0.3;

v = sqrt(((c(:, 1) - x0).^2)/(c_x^2) + ((c(:, 2) - y0).^2)/(c_y^2));

% Calculation of edge vectors directions, lengths and angles  in order to
% initialize gradient approximation parameters

[edgevectors, lengths, right_lengths, angles, omegas, index] = ...
    edgevectors_angles(n, al, c, degs);

% Calculation of gradient approximation 

grad_v = gradient_approximation_step(n, al, degs, v,...
    edgevectors, lengths, angles);

% Average filtering inside neighborhood on the approximate gradient
% vector field to smooth the result
[grad_v_av, gradmagn_v_av] = gradient_average_filter(al, degs, n, grad_v);
    
% Median filtering inside neighborhood on the approximate gradient
% vector field to smooth the result
[grad_v_med, gradmagn_v_med] = gradient_median_filter(al, degs, n, grad_v);

% Curvature approximate calculation using Riemann sum approximating
% divergence

curv_v_geom_av = curv_geom(al, degs, n, angles, omegas, right_lengths, index,...
    grad_v_av, gradmagn_v_av, 1);
curv_v_geom_med = curv_geom(al, degs, n, angles, omegas, right_lengths, index,...
    grad_v_med, gradmagn_v_med, 2);

% Curvature approximate calculation using divergence expression

curv_v_grad_av = curv_sum_pds(n, al, degs, grad_v_av, gradmagn_v_av,...
    edgevectors, lengths, angles, 1);
curv_v_grad_med = curv_sum_pds(n, al, degs, grad_v_med, gradmagn_v_med,...
    edgevectors, lengths, angles, 2);

% Analytical expression for curvature in the conic
% case, used for comparison reasons

curv_anal_v = (((c(:, 1) - x0).^2)/(c_x^4 * c_y^2) + ((c(:, 2) - y0).^2)/(c_y^4 * c_x^2))./...
    ((((c(:, 1) - x0).^2)/(c_x^4) + ((c(:, 2) - y0).^2)/(c_y^4)).^(3/2));

%% Curvature plotting

tri = delaunay(c(:, 1), c(:, 2));

% Exact curvature
figure;
trisurf(tri, c(:, 1), c(:, 2), curv_anal_v);
axis square
zlim([-7, 11]);
view([37.5, 30]);
xlabel('x');
ylabel('y');
zlabel('\kappa');
set(get(gca,'ZLabel'),'Rotation',0);
set(gca, 'CLim', [-7, 11]);
set(gca, 'ZTick', [-7, -1, 5, 11]);
h1 = gcf;
h1 = tightfig(h1);

% Approximate curvature with geometric approximation and average filtering
figure;
trisurf(tri, c(:, 1), c(:, 2), curv_v_geom_av);
axis square
zlim([-7, 11]);
view([37.5, 30]);
xlabel('x');
ylabel('y');
zlabel('\kappa');
set(get(gca,'ZLabel'),'Rotation',0);
set(gca, 'CLim', [-7, 11]);
set(gca, 'ZTick', [-7, -1, 5, 11]);
h1 = gcf;
h1 = tightfig(h1);

% Approximate curvature with geometric approximation and median filtering
figure;
trisurf(tri, c(:, 1), c(:, 2), curv_v_geom_med);
axis square
zlim([-7, 11]);
view([37.5, 30]);
xlabel('x');
ylabel('y');
zlabel('\kappa');
set(get(gca,'ZLabel'),'Rotation',0);
set(gca, 'CLim', [-7, 11]);
set(gca, 'ZTick', [-7, -1, 5, 11]);
h1 = gcf;
h1 = tightfig(h1);

% Approximate curvature with gradient-based approximation and average filtering
figure;
trisurf(tri, c(:, 1), c(:, 2), curv_v_grad_av);
axis square
zlim([-7, 11]);
view([37.5, 30]);
xlabel('x');
ylabel('y');
zlabel('\kappa');
set(get(gca,'ZLabel'),'Rotation',0);
set(gca, 'CLim', [-7, 11]);
set(gca, 'ZTick', [-7, -1, 5, 11]);
h1 = gcf;
h1 = tightfig(h1);

% Approximate curvature with gradient-based approximation and median filtering
figure;
trisurf(tri, c(:, 1), c(:, 2), curv_v_grad_med);
axis square
zlim([-7, 11]);
view([37.5, 30]);
xlabel('x');
ylabel('y');
zlabel('\kappa');
set(get(gca,'ZLabel'),'Rotation',0);
set(gca, 'CLim', [-7, 11]);
set(gca, 'ZTick', [-7, -1, 5, 11]);
h1 = gcf;
h1 = tightfig(h1);
