close all;
clear all;

%% Graph-based computation of smoothed image gradient

% Determine graph's size and radius
n = 5500;
a = 1/3;
b = 3/5;
r = b*n^(-a);
[~, al, c, degs] = grgall_small_lists(n, r);

% Delaunay triangulation of the graph for presentation purposes
tri = delaunay(c(:, 1), c(:, 2));

% Shape: circular disk
I = (sqrt((c(:, 1)-0.5).^2 + (c(:, 2)-0.5).^2) < 0.3) + 0;

% g function's parameters
sigma_old = 0.05;
lambda_old = 1000;
sigma_new1 = 0.02;
lambda_new1 = 0.05;
sigma_new2 = 0.02;
lambda_new2 = 0.05;

% Compute g function on graph with all methods
tic;
[g_old, I_s_grmag_old, I_s_old] = g_function_naive_maxapprox(I, c, n, al, degs, sigma_old, lambda_old);
toc;
tic;
[g_new1, I_s_grmag_new1, I_s_new1] = g_function_maxapprox_grad(I, c, n, al, degs, sigma_new1, lambda_new1);
toc;
tic;
[g_new2, I_s_grmag_new2] = g_function_gauss_grad_sep_nrmlz(I, c, n, al, degs, sigma_new2, lambda_new2);
toc;

%% Present results

figure;
trisurf(tri, c(:, 1), c(:, 2), I);
xlabel('x');
ylabel('y');
zlabel('I');
set(get(gca,'ZLabel'),'Rotation',0);

figure;
trisurf(tri, c(:, 1), c(:, 2), I_s_old);
xlabel('x');
ylabel('y');
zlabel('I_\sigma');
set(get(gca,'ZLabel'),'Rotation',0);
figure;
trisurf(tri, c(:, 1), c(:, 2), I_s_new1);
xlabel('x');
ylabel('y');
zlabel('I_\sigma');
set(get(gca,'ZLabel'),'Rotation',0);

figure;
trisurf(tri, c(:, 1), c(:, 2), I_s_grmag_old);
view([-37.5, 70]);
xlabel('x');
ylabel('y');
zlabel('$$\|\nabla I_\sigma\|$$', 'Interpreter', 'Latex');
set(get(gca,'ZLabel'),'Rotation',90);
figure;
trisurf(tri, c(:, 1), c(:, 2), I_s_grmag_new1);
view([-37.5, 70]);
xlabel('x');
ylabel('y');
zlabel('$$\|\nabla I_\sigma\|$$', 'Interpreter', 'Latex');
set(get(gca,'ZLabel'),'Rotation',90);
figure;
trisurf(tri, c(:, 1), c(:, 2), I_s_grmag_new2);
view([-37.5, 70]);
xlabel('x');
ylabel('y');
zlabel('$$\|\nabla I_\sigma\|$$', 'Interpreter', 'Latex');
set(get(gca,'ZLabel'),'Rotation',90);

figure;
trisurf(tri, c(:, 1), c(:, 2), g_old);
view([-37.5, 50]);
xlabel('x');
ylabel('y');
zlabel('g');
set(get(gca,'ZLabel'),'Rotation',0);
figure;
trisurf(tri, c(:, 1), c(:, 2), g_new1);
view([-37.5, 50]);
xlabel('x');
ylabel('y');
zlabel('g');
set(get(gca,'ZLabel'),'Rotation',0);
figure;
trisurf(tri, c(:, 1), c(:, 2), g_new2);
view([-37.5, 50]);
xlabel('x');
ylabel('y');
zlabel('g');
set(get(gca,'ZLabel'),'Rotation',0);

