function phi_0 = dist_given_contour(contour_internal, c, al, n, degs)

% Inputs:
%   contour_internal    - nx1 logical vector determining which of the
%                       graph's vertices belong to the given initial
%                       contour's internal
%   c                   - nx2 matrix holding the positions of the vertices
%   al                  - adjacency list of the graph stored as an nxm
%                       matrix, where m is maximal vertex degree on the
%                       graph
%   n                   - number of graph vertices
%   degs                - 1xn vector holding the degree of each vertex

% Outputs:
%   phi_0   - nx1 vector with approximate signed distance transform from
%           the initial contour

% Compute the inner distance transform of a contour on a graph whose
% vertices have coordinates c

% contour_inside and contour_outside are logical vectors determining which
% of the graph's vertices belong to the initial contour's inner and outer
% vertex sets

contour_external = ~contour_internal;

external_dilation = contour_external;
for i=1:n
    if (contour_external(i))
        external_dilation(al(i, 1:degs(i))) = 1;
    end
end
contour_inside = logical(external_dilation - contour_external);

contour_outside = zeros(n, 1);
for i=1:n
    if (contour_inside(i))
        contour_outside(al(i, 1:degs(i))) = 1;
    end
end
contour_outside = (contour_outside - contour_internal > 0);

% Compute squared distance of every node in the graph from every node on
% the contour's inner vertex set
[xs_contour, xs_all] = meshgrid(c(contour_inside, 1), c(:, 1));
[ys_contour, ys_all] = meshgrid(c(contour_inside, 2), c(:, 2));
squared_distances_inside = (xs_all - xs_contour).^2 + (ys_all - ys_contour).^2;

% Compute squared distance of every node in the graph from every node on
% the contour's outer vertex set
[xs_contour, xs_all] = meshgrid(c(contour_outside, 1), c(:, 1));
[ys_contour, ys_all] = meshgrid(c(contour_outside, 2), c(:, 2));
squared_distances_outside = (xs_all - xs_contour).^2 + (ys_all - ys_contour).^2;

% Apply the minimum operator to assign values to the distance transform
phi_0 = (sqrt(min(squared_distances_inside, [], 2)) +...
    sqrt(min(squared_distances_outside, [], 2)))/2;
% Modify the sign of the result for the contour's external
phi_0(contour_external) = - phi_0(contour_external);


end