function [g, I_s_grmag, I_s] = g_function_naive_maxapprox(I, c, n, al, degs,...
    sigma, lambda)

% Compute distance of every pair of nodes in the graph in distances matrix
[xs1, xs2] = meshgrid(c(:, 1), c(:, 1));
[ys1, ys2] = meshgrid(c(:, 2), c(:, 2));
squared_distances = (xs1 - xs2).^2 + (ys1 - ys2).^2;

% Gaussian gradient smoothing function
gauss = exp(-squared_distances/(2*sigma^2))/(2*pi*sigma^2);

% Compute smoothed image
I_s = gauss*I;

% Compute smoothed image's gradient with old faithful gradient approximation
I_s_grmag = gradmagn_max(n, al, degs, I_s);

% Filter the results with median filter to reduce the error
I_s_grmag = median_filter(al, degs, n, I_s_grmag);

% Compute g function with smoothed image gradient as its argument using
% analytical form in Drakopoulos' - Maragos' paper (Caselles-Catte-Coll-
% Dibos and Malladi-Sethian-Vemuri proposal)
g = (1 + (I_s_grmag.^2)/(lambda^2)).^(-1);

end