function gradmagn = gradmagn_max(n, al, degs, u)

% Inputs:
%   n           - number of graph vertices
%   al          - adjacency list of the graph stored as an nxm matrix,
%               where m is maximal vertex degree on the graph
%   degs        - 1xn vector holding the degree of each vertex
%   u           - real function on graph vertices as an nx1 vector

% Outputs:
%   gradmagn    - nx1 vector with approximate magnitude of gradient field

% Compute the approximate gradient magnitude of a function defined on a
% graph using the maximum filter on the unit-scale Vincent neighborhood

% Add every vertex to its adjacent list so that its value of u is taken
% into account in the median filtering
degs = degs + 1;
al = [al zeros(n, 1)];
for i=1:n
    al(i, degs(i)) = i;
end

gradmagn = zeros(n, 1);

% Main maximum filtering loop
for i=1:n
    gradmagn(i) = max(abs(u(al(i, 1:degs(i))) - u(i)));
end

end

