function [ am, al, c, degs ] = grgall_small_lists( n, r )

% Inputs:
%   n       - number of graph vertices
%   r       - graph radius

% Outputs:
%   am      - nxn adjacency matrix of the created graph
%   al      - adjacency list of the created graph stored as an nxm matrix,
%           where m is maximal vertex degree on the graph
%   c       - nx2 matrix holding the positions of the vertices
%   degs    - 1xn vector holding the degree of each vertex

% Assigning random positions to the nodes of a geometric random graph
% inside the unit square

c = rand(n, 2);

% Defining the adjacency matrix and lists of the geometric random graph

adjacency = zeros(n);
adjacentlists = zeros(n);
degrees = zeros(1, n);
for i=1:(n-1)
    for j=(i+1):n
        distance = ((c(i, 1) - c(j, 1))^2 + (c(i, 2) - c(j, 2))^2)^0.5;
        if (distance <= r)
            adjacency(i, j) = 1;
            adjacency(j, i) = 1;
            degrees(i) = degrees(i)+1;
            degrees(j) = degrees(j)+1;
            adjacentlists(i, degrees(i)) = j;
            adjacentlists(j, degrees(j)) = i;
        end
    end
end

am = adjacency;
al = adjacentlists(:, 1:max(degrees));
degs = degrees;

end

