function u_max = max_filter(al, degs, n, u)

% Inputs:
%   al      - adjacency list of the graph stored as an nxm matrix,
%           where m is maximal vertex degree on the graph
%   degs    - 1xn vector holding the degree of each vertex
%   n       - number of graph vertices
%   u       - initial real function on graph vertices as an nx1 vector

% Outputs:
%   u_max   - nx1 vector with filtered function

% Add every vertex to its adjacent list so that its value of u is taken
% into account in the median filtering
degs = degs + 1;
al = [al zeros(n, 1)];
for i=1:n
    al(i, degs(i)) = i;
end

u_max = zeros(n, 1);

% Main maximum filtering loop
for i=1:n
    u_max(i) = max(u(al(i, 1:degs(i))));
end

end

